/**
Copyright (C) 2004, 2005, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include <windows.h>
#include "qstring.h"

QString date("29.12.2005");

int main(int argc, char **argv) {
  // Required for accessing registry keys
  HKEY hRegRoot = HKEY_LOCAL_MACHINE;  // Registry Root handle
  HKEY hRegKey;                        // Registry Key handle
  DWORD dwType = REG_SZ;               // Data type
  DWORD cbData = 255;                  // Buffer length
  DWORD retCode;                       // Function return code

  char* v_val = (char*)malloc(255), *check;
  QString line("                                                                                                                                                                                                                                                                                            ");
  QString upddate, tempStr;
  QString drvpos;   // driver position in registry
  bool error=false;
  int c;
  FILE *fin, *fout;

  // printf("%d |%s|\n", argc, argv[1]);
  
  if (argc>2) {
    puts("You only could pass maximally one parameter. Press RETURN to exit program...");
    getchar();
	exit(1);
  }
  if (argc==2) {
    tempStr=argv[1];
    if (tempStr!="/e") {
  	  puts("Unknown parameter passed. Press RETURN to exit program...");
      getchar();
      exit(1);
    }
    else {
      error=true;
    }
  }

  if (!(fin=fopen("kyro.txt","r"))) {
	puts("Could not open kyro.txt to read. Press RETURN to exit program...");
    getchar();
	exit(1);
  }
  do {
    c=0;
    check=fgets(line.data(),256,fin);          // reading line
    c=line.free2();
    if (!check) c=-1;               // end of file
    if (!c) {
      if (line.left(line.find(':'))=="Date") {  // Date of update
        upddate=line.right(line.length()-line.find(':')-2);
        break;
      }
    }
  } while (check);
  fclose(fin);

  puts ("Updates and creates new game profiles for PowerVR-Kyro graphic cards\n");
  puts ("Written by Dominik Wagenfuehr\nErrors and comments to dominik.wagenfuehr@arcor.de");
  printf ("Base file date: %s\n", date.data());
  printf ("Registry file date: %s\n\n", upddate.data());
  
  if (RegOpenKeyEx(hRegRoot,"SOFTWARE\\PowerVR\\KYRO",0,KEY_READ,&hRegKey)!=0) {
    puts ("It seems you don't have installed a PowerVR-Kyro graphic cards in your system.");
    puts ("If this isn't right please contact dominik@cyskat.de\n");
	puts ("Press RETURN to exit program...");
    getchar();
	exit(1);
  }
  if (RegQueryValueEx(hRegKey, "LatestDrvReg", NULL, &dwType, (unsigned char*)v_val, &cbData)!=0) {
    puts ("There was an error extracting the registry value.");
    puts ("Please contact dominik@cyskat.de\n");
	puts ("Press RETURN to exit program...");
    getchar();
	exit(1);
  }
  RegCloseKey(hRegRoot);

  drvpos="HKEY_LOCAL_MACHINE\\";
  drvpos+=v_val;
  drvpos+="\\KYRO\\Game Settings";
  // printf("%s\n",drvpos.data());  getchar();

  if (!(fin=fopen("kyro.txt","r"))) {
		puts("Could not open kyro.txt to read. Press RETURN to exit program...");
        getchar();
		exit(1);
	}
  	if (!(fout=fopen("upd-kyro.reg","w"))) {
		puts("Could not open upd-kyro.reg to write. Press RETURN to exit program...");
		fclose(fin);
		exit(1);
	}

	fputs("REGEDIT4\n",fout);

    QString name="";
    QString typ="";
    QString exe="";
    QString key="";

    do {
      c=0;
      check=fgets(line.data(),256,fin);          // reading line
      c=line.free2();
      if (!check) c=-1;               // end of file
      if (!c) {
        tempStr="";
        if (line.left(line.find(':'))=="Name") {  // Name of the game
          name=line.right(line.length()-line.find(':')-2);
          tempStr="\n; ";
          tempStr+=name;
          fputs(tempStr.data(),fout);
        }
        else if (line.left(line.find(':'))=="Typ") { // typ of game (OGL, D3D, DDRAW)
          typ=line.right(line.length()-line.find(':')-2);
          tempStr="\n[";
          tempStr+=drvpos;
          tempStr+="\\";
          tempStr+=typ;
          tempStr+="\\";
          fputs(tempStr.data(),fout);
        }
        else if (line.left(line.find(':'))=="Exe") {  // file name
          exe=line.right(line.length()-line.find(':')-2);
          tempStr=exe;
          tempStr+="]\n";
          fputs(tempStr.data(),fout);
          tempStr="\"UIDummyName\"=\"";
          tempStr+=name;
          tempStr+="\"\n";
          fputs(tempStr.data(),fout);
        }
        else if (line.length()>0 && line.find(':')==-1) { // registry settings
          key=line;
          tempStr+=key;
          tempStr+="\n";
          fputs(tempStr.data(),fout);
        }
      }
	} while (check);
    fclose(fout);
    fclose(fin);

    puts("Update-File created.\n");
    puts("New registry keys will be imported.\nPlease press \"Yes\" in the window\n\nand then press RETURN to exit programme...");
    system("regedit.exe upd-kyro.reg");
    getchar();

    if (!error) remove ("upd-kyro.reg");

    return (0);
}
